@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2015 DNAnexus, Inc.
+# Copyright (C) 2013-2016 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -28,7 +28,7 @@ from __future__ import print_function, unicode_literals, division, absolute_impo
 
 import dxpy
 from . import DXDataObject, DXJob
-from ..utils import merge
+from ..utils import merge, instance_type_to_sys_reqs
 from ..exceptions import DXError
 from ..compat import basestring
 
@@ -41,17 +41,6 @@ class DXExecutable:
     def __init__(self, *args, **kwargs):
         raise NotImplementedError("This class is a mix-in. Use DXApp or DXApplet instead.")
 
-    @staticmethod
-    def _inst_type_to_sys_reqs(instance_type):
-        if isinstance(instance_type, basestring):
-            # All entry points should use this instance type
-            return {"*": {"instanceType": instance_type}}
-        elif isinstance(instance_type, dict):
-            # Map of entry point to instance type
-            return {fn: {"instanceType": fn_inst} for fn, fn_inst in instance_type.items()}
-        else:
-            raise DXError('Expected instance_type field to be either a string or a dict')
-
     @staticmethod
     def _get_run_input_common_fields(executable_input, **kwargs):
         '''
@@ -67,7 +56,7 @@ class DXExecutable:
                 run_input[arg] = kwargs[arg]
 
         if kwargs.get('instance_type') is not None:
-            run_input["systemRequirements"] = DXExecutable._inst_type_to_sys_reqs(kwargs['instance_type'])
+            run_input["systemRequirements"] = instance_type_to_sys_reqs(kwargs['instance_type'])
 
         if kwargs.get('depends_on') is not None:
             run_input["dependsOn"] = []
@@ -135,6 +124,30 @@ class DXExecutable:
         """
         raise NotImplementedError('_get_run_input is not implemented')
 
+    def _get_required_keys(self):
+        """
+        Abstract method used in executable_unbuilder.dump_executable
+        """
+        raise NotImplementedError('_get_required_keys is not implemented')
+
+    def _get_optional_keys(self):
+        """
+        Abstract method used in executable_unbuilder.dump_executable
+        """
+        raise NotImplementedError('_get_optional_keys is not implemented')
+
+    def _get_describe_output_keys(self):
+        """
+        Abstract method used in executable_unbuilder.dump_executable
+        """
+        raise NotImplementedError('_get_describe_output_keys is not implemented')
+
+    def _get_cleanup_keys(self):
+        """
+        Abstract method used in executable_unbuilder.dump_executable
+        """
+        raise NotImplementedError('_get_cleanup_keys is not implemented')
+
     def run(self, executable_input, project=None, folder=None, name=None, tags=None, properties=None, details=None,
             instance_type=None, stage_instance_types=None, stage_folders=None, rerun_stages=None,
             depends_on=None, allow_ssh=None, debug=None, delay_workspace_destruction=None, priority=None,
@@ -203,6 +216,13 @@ class DXExecutable:
 ############
 # DXApplet #
 ############
+_applet_required_keys = ['name', 'title', 'summary', 'types', 'tags',
+                         'properties', 'dxapi', 'inputSpec', 'outputSpec',
+                         'runSpec', 'access', 'details']
+_applet_optional_keys = []
+_applet_describe_output_keys = ['properties', 'details']
+_applet_cleanup_keys = ['name', 'title', 'summary', 'types', 'tags',
+                        'properties', 'runSpec', 'access', 'details']
 
 def _makeNonexistentAPIWrapper(method):
     def nonexistentAPIWrapper(object_id, input_params=None, always_retry=None, **kwargs):
@@ -335,6 +355,18 @@ class DXApplet(DXDataObject, DXExecutable):
     def _get_run_input(self, executable_input, **kwargs):
         return DXExecutable._get_run_input_fields_for_applet(executable_input, **kwargs)
 
+    def _get_required_keys(self):
+        return _applet_required_keys
+
+    def _get_optional_keys(self):
+        return _applet_optional_keys
+
+    def _get_describe_output_keys(self):
+        return _applet_describe_output_keys
+
+    def _get_cleanup_keys(self):
+        return _applet_cleanup_keys
+
     def run(self, applet_input, *args, **kwargs):
         """
         Creates a new job that executes the function "main" of this applet with
