@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2015 DNAnexus, Inc.
+# Copyright (C) 2013-2016 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -24,8 +24,11 @@ import os, json, collections, concurrent.futures, traceback, sys, time, gc
 from multiprocessing import cpu_count
 import dateutil.parser
 from .. import logger
-from ..compat import basestring
-
+from ..compat import basestring, THREAD_TIMEOUT_MAX
+from ..exceptions import DXError
+import numbers
+import binascii
+import random
 
 def _force_quit(signum, frame):
     # traceback.print_stack(frame)
@@ -42,7 +45,7 @@ def wait_for_a_future(futures, print_traceback=False):
     """
     while True:
         try:
-            future = next(concurrent.futures.as_completed(futures, timeout=sys.maxint))
+            future = next(concurrent.futures.as_completed(futures, timeout=THREAD_TIMEOUT_MAX))
             break
         except concurrent.futures.TimeoutError:
             pass
@@ -82,21 +85,30 @@ def wait_for_all_futures(futures, print_traceback=False):
         os._exit(os.EX_IOERR)
 
 
-def response_iterator(request_iterator, thread_pool, max_active_tasks=None):
+def response_iterator(request_iterator, thread_pool, max_active_tasks=None, do_first_task_sequentially=True):
     """
-    :param request_iterator: An iterator producing inputs for consumption by the worker pool.
+    :param request_iterator:
+        An iterator producing inputs for consumption by the worker pool.
     :type request_iterator: iterator of callable, args, kwargs
     :param thread_pool: thread pool to submit the requests to
     :type thread_pool: concurrent.futures.thread.ThreadPoolExecutor
     :param max_active_tasks:
-        The maximum number of tasks that may be either running or waiting for consumption of their result.
-        If not given, defaults to the number of CPU cores on the machine.
+        The maximum number of tasks that may be either running or
+        waiting for consumption of their result. If not given, defaults
+        to the number of CPU cores on the machine.
     :type max_active_tasks: int
+    :param do_first_task_sequentially:
+        If True, executes (and returns the result of) the first request
+        before submitting any other requests (the subsequent requests
+        are submitted with *max_active_tasks* parallelism).
+    :type do_first_task_sequentially: bool
+
+    Rate-limited asynchronous multithreaded task runner. Consumes tasks
+    from *request_iterator*. Yields their results in order, while
+    allowing up to *max_active_tasks* to run simultaneously. Unlike
+    concurrent.futures.Executor.map, prevents new tasks from starting
+    while there are *max_active_tasks* or more unconsumed results.
 
-    Rate-limited asynchronous multithreaded task runner.
-    Consumes tasks from *request_iterator*. Yields their results in order, while allowing up to *max_active_tasks* to run
-    simultaneously. Unlike concurrent.futures.Executor.map, prevents new tasks from starting while there are
-    *max_active_tasks* or more unconsumed results.
     """
     tasks_in_progress = collections.deque()
     if max_active_tasks is None:
@@ -111,12 +123,16 @@ def response_iterator(request_iterator, thread_pool, max_active_tasks=None):
     def next_result(tasks_in_progress):
         future = tasks_in_progress.popleft()
         try:
-            result = future.result(timeout=sys.maxint)
+            result = future.result(timeout=THREAD_TIMEOUT_MAX)
         except KeyboardInterrupt:
             print('')
             os._exit(os.EX_IOERR)
         return result
 
+    if do_first_task_sequentially:
+        task_callable, task_args, task_kwargs = next(request_iterator)
+        yield task_callable(*task_args, **task_kwargs)
+
     for _i in range(max_active_tasks):
         try:
             submit_task(request_iterator, thread_pool, tasks_in_progress)
@@ -166,9 +182,10 @@ def normalize_time_input(t, future=False, default_unit='ms'):
         except ValueError:
             try:
                 t = int(time.mktime(dateutil.parser.parse(t).timetuple())*1000)
-            except ValueError:
+                assert t > 0
+            except (ValueError, OverflowError, AssertionError):
                 raise ValueError(error_msg.format(t=t))
-    elif isinstance(t, int):
+    elif isinstance(t, numbers.Integral):
         units_multipliers = {'ms': 1, 's': 1000}
         if default_unit not in units_multipliers:
             raise ValueError("Expected default_unit to be one of 's' or 'ms'")
@@ -269,5 +286,55 @@ def json_loads_raise_on_duplicates(*args, **kwargs):
 def warn(*args, **kwargs):
     print(*args, file=sys.stderr, **kwargs)
 
+
+def instance_type_to_sys_reqs(instance_type, entrypoint="*"):
+    """
+    Returns a dictionary that can be passed as a "systemRequirements" input
+    to job/new or run/ API calls. The instance_type should be either a:
+    * string, eg. mem1_ssd1_x2
+    * dictionary, eg. {"main": "mem2_hdd2_x2", "other_function": "mem2_hdd2_x1"}
+    """
+    if isinstance(instance_type, basestring):
+        # By default, all entry points ("*") should use this instance type
+        return {entrypoint: {"instanceType": instance_type}}
+    elif isinstance(instance_type, dict):
+        # instance_type is a map of entry point to instance type
+        return {fn: {"instanceType": fn_inst} for fn, fn_inst in instance_type.items()}
+    else:
+        raise DXError('Expected instance_type field to be either a string or a dict')
+
+class Nonce:
+    '''
+    Generates a nonce by using the system's random number generator. If it fails
+    it uses python's random library to generate a random long integer.
+    The nonce is the random number concatenated with the time.
+    '''
+    def __init__(self):
+        try:
+            self.nonce = "%s%f" % (str(binascii.hexlify(os.urandom(32))), time.time())
+        except:
+            random.seed(time.time())
+            self.nonce = "%s%f" % (str(random.getrandbits(8*26)), time.time())
+
+    def __str__(self):
+        return self.nonce
+
+    @staticmethod
+    def update_nonce(input_params):
+        '''
+        Static method to return a copy of the input dictionary with an
+        additional unique nonce
+        :param input: an input dictionary that may be empty
+        :type input: dict
+        :returns an extended copy of the input with an additional nonce field
+
+        The input dictionary is updated with a nonce only if does not already
+        have a non empty nonce
+        '''
+        input_cp = input_params.copy()
+        if len(input_cp.get('nonce', '')) == 0:
+            input_cp['nonce'] = str(Nonce())
+        return input_cp
+
 # Moved to the bottom due to circular imports
 from .exec_utils import run, convert_handlers_to_dxlinks, parse_args_as_job_input, entry_point, DXJSONEncoder
