@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2013-2015 DNAnexus, Inc.
+# Copyright (C) 2013-2016 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -25,8 +25,10 @@ containers, dataobjects, apps, and jobs).
 from __future__ import print_function, unicode_literals, division, absolute_import
 
 import datetime, time, json, math, sys, copy
+import subprocess
 from collections import defaultdict
 
+import dxpy
 from .printing import (RED, GREEN, BLUE, YELLOW, WHITE, BOLD, UNDERLINE, ENDC, DELIMITER, get_delimiter, fill)
 from ..compat import basestring
 
@@ -54,14 +56,21 @@ def DATA_STATES(state):
 
 SIZE_LEVEL = ['bytes', 'KB', 'MB', 'GB', 'TB']
 
+
 def get_size_str(size):
+    """
+    Formats a byte size as a string.
+
+    The returned string is no more than 9 characters long.
+    """
     if size == 0:
         magnitude = 0
         level = 0
     else:
         magnitude = math.floor(math.log(size, 10))
         level = int(min(math.floor(magnitude // 3), 4))
-    return ('%d' if level == 0 else '%.2f') % (float(size) // 2**(level*10)) + ' ' + SIZE_LEVEL[level]
+    return ('%d' if level == 0 else '%.2f') % (float(size) / 2**(level*10)) + ' ' + SIZE_LEVEL[level]
+
 
 def parse_typespec(thing):
     if isinstance(thing, basestring):
@@ -118,6 +127,8 @@ def get_io_desc(parameter, include_class=True, show_opt=True, app_help_version=F
     return desc
 
 def get_io_spec(spec, skip_fields=None):
+    if spec is None:
+        return 'null'
     if skip_fields is None:
         skip_fields = []
     filtered_spec = [param for param in spec if param["name"] not in skip_fields]
@@ -240,30 +251,36 @@ def job_output_to_str(job_output, prefix='\n', title="Output: ", title_len=None)
                                                                    subsequent_indent=' '*9,
                                                                    break_long_words=False) for key, value in job_output.items()])
 
+
 def get_io_field(io_hash, defaults=None, delim='=', highlight_fields=()):
+
+    def highlight_value(key, value):
+        if key in highlight_fields:
+            return YELLOW() + value + ENDC()
+        else:
+            return value
+
     if defaults is None:
         defaults = {}
     if io_hash is None:
         return '-'
     if len(io_hash) == 0 and len(defaults) == 0:
         return '-'
-    def highlight_value(key, value):
-        if key in highlight_fields:
-            return YELLOW() + value + ENDC()
-        else:
-            return value
     if get_delimiter() is not None:
         return ('\n' + get_delimiter()).join([(key + delim + highlight_value(key, io_val_to_str(value))) for key, value in io_hash.items()] +
                                              [('[' + key + delim + io_val_to_str(value) + ']') for key, value in defaults.items()])
     else:
-        return ('\n').join([fill(key + ' ' + delim + ' ' + highlight_value(key, io_val_to_str(value)),
-                                 initial_indent=' '*16,
-                                 subsequent_indent=' '*17,
-                                 break_long_words=False) for key, value in io_hash.items()] +
-                           [fill('[' + key + ' ' + delim + ' ' + io_val_to_str(value) + ']',
-                                 initial_indent=' '*16,
-                                 subsequent_indent=' '*17,
-                                 break_long_words=False) for key, value in defaults.items()])[16:]
+        lines = [fill(key + ' ' + delim + ' ' + highlight_value(key, io_val_to_str(value)),
+                      initial_indent=' ' * FIELD_NAME_WIDTH,
+                      subsequent_indent=' ' * (FIELD_NAME_WIDTH + 1),
+                      break_long_words=False)
+                 for key, value in io_hash.items()]
+        lines.extend([fill('[' + key + ' ' + delim + ' ' + io_val_to_str(value) + ']',
+                           initial_indent=' ' * FIELD_NAME_WIDTH,
+                           subsequent_indent=' ' * (FIELD_NAME_WIDTH + 1),
+                           break_long_words=False)
+                      for key, value in defaults.items()])
+        return '\n'.join(lines)[FIELD_NAME_WIDTH:]
 
 def get_resolved_jbors(resolved_thing, orig_thing, resolved_jbors):
     if resolved_thing == orig_thing:
@@ -294,7 +311,26 @@ def get_resolved_jbors(resolved_thing, orig_thing, resolved_jbors):
             get_resolved_jbors(resolved_thing[key], orig_thing[key], resolved_jbors)
 
 def render_bundleddepends(thing):
-    return [item["name"] + " (" + item["id"]["$dnanexus_link"] + ")" for item in thing]
+    from ..bindings.search import find_one_data_object
+    from ..exceptions import DXError
+    bundles = []
+    for item in thing:
+        bundle_asset_record = dxpy.DXFile(item["id"]["$dnanexus_link"]).get_properties().get("AssetBundle")
+        asset = None
+
+        if bundle_asset_record:
+            asset = dxpy.DXRecord(bundle_asset_record)
+
+        if asset:
+            try:
+                bundles.append(asset.describe().get("name") + " (" + asset.get_id() + ")")
+            except DXError:
+                asset = None
+
+        if not asset:
+            bundles.append(item["name"] + " (" + item["id"]["$dnanexus_link"] + ")")
+
+    return bundles
 
 def render_execdepends(thing):
     rendered = []
@@ -408,7 +444,7 @@ def print_project_desc(desc, verbose=False):
     if 'containsPHI' in desc:
         print_json_field('Contains PHI', desc['containsPHI'])
     if 'archivalState' in desc and verbose:
-        print_json_field('Archival state', desc['archivalState'])
+        print_field('Archival state', desc['archivalState'])
     if 'archivalProgress' in desc and verbose:
         print_json_field('Archival progress', desc['archivalProgress'])
 
@@ -423,9 +459,9 @@ def print_project_desc(desc, verbose=False):
     if 'storageCost' in desc:
         print_field("Storage cost", "$%.3f/month" % desc["storageCost"])
     if 'totalSponsoredEgressBytes' in desc or 'consumedSponsoredEgressBytes' in desc:
-        total_egress_str = '%.2f GB' % (desc['totalSponsoredEgressBytes'] // 1073741824.,) \
+        total_egress_str = '%.2f GB' % (desc['totalSponsoredEgressBytes'] / 1073741824.,) \
                            if 'totalSponsoredEgressBytes' in desc else '??'
-        consumed_egress_str = '%.2f GB' % (desc['consumedSponsoredEgressBytes'] // 1073741824.,) \
+        consumed_egress_str = '%.2f GB' % (desc['consumedSponsoredEgressBytes'] / 1073741824.,) \
                               if 'consumedSponsoredEgressBytes' in desc else '??'
         print_field('Sponsored egress',
                     ('%s used of %s total' % (consumed_egress_str, total_egress_str)))
@@ -466,6 +502,11 @@ def print_project_desc(desc, verbose=False):
         if field not in recognized_fields:
             print_json_field(field, desc[field])
 
+def get_advanced_inputs(desc, verbose):
+    details = desc.get("details")
+    if not verbose and isinstance(details, dict):
+        return details.get("advancedInputs", [])
+    return []
 
 def print_app_desc(desc, verbose=False):
     recognized_fields = ['id', 'class', 'name', 'version', 'aliases', 'createdBy', 'created', 'modified', 'deleted', 'published', 'title', 'subtitle', 'description', 'categories', 'access', 'dxapi', 'inputSpec', 'outputSpec', 'runSpec', 'resources', 'billTo', 'installed', 'openSource', 'summary', 'applet', 'installs', 'billing', 'details', 'developerNotes',
@@ -530,18 +571,75 @@ def print_app_desc(desc, verbose=False):
         if field not in recognized_fields:
             print_json_field(field, desc[field])
 
+def print_globalworkflow_desc(desc, verbose=False):
+    recognized_fields = ['id', 'class', 'name', 'version', 'aliases', 'createdBy', 'created',
+                         'modified', 'deleted', 'published', 'title', 'description',
+                         'categories', 'dxapi', 'billTo', 'summary', 'billing', 'developerNotes',
+                         'authorizedUsers', 'regionalOptions']
+
+    print_field("ID", desc["id"])
+    print_field("Class", desc["class"])
+    if 'billTo' in desc:
+        print_field("Billed to", desc['billTo'][5 if desc['billTo'].startswith('user-') else 0:])
+    print_field("Name", desc["name"])
+    print_field("Version", desc["version"])
+    print_list_field("Aliases", desc["aliases"])
+    print_field("Created by", desc["createdBy"][5 if desc['createdBy'].startswith('user-') else 0:])
+    print_field("Created", render_timestamp(desc['created']))
+    print_field("Last modified", render_timestamp(desc['modified']))
+    # print_json_field('Open source', desc['openSource'])
+    # print_json_field('Deleted', desc['deleted'])
+    if not desc.get('deleted', False):
+        advanced_inputs = []
+        if 'published' not in desc or desc["published"] < 0:
+            print_field("Published", "-")
+        else:
+            print_field("Published", render_timestamp(desc['published']))
+        if "title" in desc and desc['title'] is not None:
+            print_field("Title", desc["title"])
+        if "subtitle" in desc and desc['subtitle'] is not None:
+            print_field("Subtitle", desc["subtitle"])
+        if 'summary' in desc and desc['summary'] is not None:
+            print_field("Summary", desc['summary'])
+        print_list_field("Categories", desc["categories"])
+        if 'details' in desc:
+            print_json_field("Details", desc["details"])
+        print_field("API version", desc["dxapi"])
+
+        # Additionally, print inputs, outputs, stages of the underlying workflow
+        # from the region of the current workspace
+        current_project = dxpy.WORKSPACE_ID
+        if current_project:
+            region = dxpy.api.project_describe(current_project, input_params={"fields": {"region": True}})["region"]
+            if region and region in desc['regionalOptions']:
+                workflow_desc = desc['regionalOptions'][region]['workflowDescribe']
+                print_field("Workflow region", region)
+                if 'id' in workflow_desc:
+                    print_field("Workflow ID", workflow_desc['id'])
+                if workflow_desc.get('inputSpec') is not None and workflow_desc.get('inputs') is None:
+                    print_nofill_field("Input Spec", get_io_spec(workflow_desc['inputSpec'], skip_fields=get_advanced_inputs(workflow_desc, verbose)))
+                if workflow_desc.get('outputSpec') is not None and workflow_desc.get('outputs') is None:
+                    print_nofill_field("Output Spec", get_io_spec(workflow_desc['outputSpec']))
+                if  workflow_desc.get('inputs') is not None:
+                    print_nofill_field("Workflow Inputs", get_io_spec(workflow_desc['inputs']))
+                if  workflow_desc.get('outputs') is not None:
+                    print_nofill_field("Workflow Outputs", get_io_spec(workflow_desc['outputs']))
+                if 'stages' in workflow_desc:
+                    for i, stage in enumerate(workflow_desc["stages"]):
+                        render_stage("Stage " + str(i), stage)
+    if 'authorizedUsers' in desc:
+        print_list_field('AuthorizedUsers', desc["authorizedUsers"])
+
+    for field in desc:
+        if field not in recognized_fields:
+            print_json_field(field, desc[field])
+
 def get_col_str(col_desc):
     return col_desc['name'] + DELIMITER(" (") + col_desc['type'] + DELIMITER(")")
 
 def print_data_obj_desc(desc, verbose=False):
     recognized_fields = ['id', 'class', 'project', 'folder', 'name', 'properties', 'tags', 'types', 'hidden', 'details', 'links', 'created', 'modified', 'state', 'title', 'subtitle', 'description', 'inputSpec', 'outputSpec', 'runSpec', 'summary', 'dxapi', 'access', 'createdBy', 'summary', 'sponsored', 'developerNotes',
-                         'stages', 'latestAnalysis', 'editVersion', 'outputFolder', 'initializedFrom']
-
-    def get_advanced_inputs():
-        details = desc.get("details")
-        if not verbose and isinstance(details, dict):
-            return details.get("advancedInputs", [])
-        return []
+                         'stages', 'inputs', 'outputs', 'latestAnalysis', 'editVersion', 'outputFolder', 'initializedFrom']
 
     print_field("ID", desc["id"])
     print_field("Class", desc["class"])
@@ -588,10 +686,18 @@ def print_data_obj_desc(desc, verbose=False):
         print_json_field("Access", desc["access"])
     if 'dxapi' in desc:
         print_field("API version", desc["dxapi"])
-    if "inputSpec" in desc and desc['inputSpec'] is not None:
-        print_nofill_field("Input Spec", get_io_spec(desc['inputSpec'], skip_fields=get_advanced_inputs()))
-    if "outputSpec" in desc and desc['outputSpec'] is not None:
+
+    # In case of a workflow: do not display "Input/Output Specs" that show stages IO
+    # when the workflow has workflow-level input/output fields defined.
+    if desc.get('inputSpec') is not None and desc.get('inputs') is None:
+        print_nofill_field("Input Spec", get_io_spec(desc['inputSpec'], skip_fields=get_advanced_inputs(desc, verbose)))
+    if desc.get('outputSpec') is not None and desc.get('outputs') is None:
         print_nofill_field("Output Spec", get_io_spec(desc['outputSpec']))
+    if  desc.get('inputs') is not None:
+        print_nofill_field("Workflow Inputs", get_io_spec(desc['inputs']))
+    if  desc.get('outputs') is not None:
+        print_nofill_field("Workflow Outputs", get_io_spec(desc['outputs']))
+
     if 'runSpec' in desc:
         print_field("Interpreter", desc["runSpec"]["interpreter"])
         if "resources" in desc['runSpec']:
@@ -642,8 +748,19 @@ def print_data_obj_desc(desc, verbose=False):
             else: # Unhandled prettifying
                 print_json_field(field, desc[field])
 
+
+def printable_ssh_host_key(ssh_host_key):
+    try:
+        keygen = subprocess.Popen(["ssh-keygen", "-lf", "/dev/stdin"], stdin=subprocess.PIPE, stdout=subprocess.PIPE)
+        (stdout, stderr) = keygen.communicate(ssh_host_key)
+    except:
+        return ssh_host_key.strip()
+    else:
+        return stdout.replace(" no comment", "").strip()
+
+
 def print_execution_desc(desc):
-    recognized_fields = ['id', 'class', 'project', 'workspace',
+    recognized_fields = ['id', 'class', 'project', 'workspace', 'region',
                          'app', 'applet', 'executable', 'workflow',
                          'state',
                          'rootExecution', 'parentAnalysis', 'parentJob', 'originJob', 'analysis', 'stage',
@@ -653,7 +770,7 @@ def print_execution_desc(desc):
                          'name', 'instanceType', 'systemRequirements', 'executableName', 'failureFrom', 'billTo',
                          'startedRunning', 'stoppedRunning', 'stateTransitions',
                          'delayWorkspaceDestruction', 'stages', 'totalPrice', 'isFree', 'invoiceMetadata',
-                         'priority']
+                         'priority', 'sshHostKey']
 
     print_field("ID", desc["id"])
     print_field("Class", desc["class"])
@@ -662,6 +779,8 @@ def print_execution_desc(desc):
     if "executableName" in desc and desc['executableName'] is not None:
         print_field("Executable name", desc['executableName'])
     print_field("Project context", desc["project"])
+    if 'region' in desc:
+        print_field("Region", desc["region"])
     if 'billTo' in desc:
         print_field("Billed to",  desc['billTo'][5 if desc['billTo'].startswith('user-') else 0:])
     if 'workspace' in desc:
@@ -671,6 +790,8 @@ def print_execution_desc(desc):
         print_field('Resources', desc['resources'])
     if "app" in desc:
         print_field("App", desc["app"])
+    elif desc.get("executable", "").startswith("globalworkflow"):
+        print_field("Workflow", desc["executable"])
     elif "applet" in desc:
         print_field("Applet", desc["applet"])
     elif "workflow" in desc:
@@ -784,9 +905,11 @@ def print_execution_desc(desc):
                 else:
                     print_nofill_field(" sys reqs", YELLOW() + json.dumps(cloned_sys_reqs) + ENDC())
     if not desc.get('isFree') and desc.get('totalPrice') is not None:
-        print_field('Total Price', "%.2f" % desc['totalPrice'])
+        print_field('Total Price', "$%.2f" % desc['totalPrice'])
     if desc.get('invoiceMetadata'):
         print_json_field("Invoice Metadata", desc['invoiceMetadata'])
+    if desc.get('sshHostKey'):
+        print_nofill_field("SSH Host Key", printable_ssh_host_key(desc['sshHostKey']))
 
     for field in desc:
         if field not in recognized_fields:
@@ -821,6 +944,8 @@ def print_desc(desc, verbose=False):
         print_project_desc(desc, verbose=verbose)
     elif desc['class'] == 'app':
         print_app_desc(desc, verbose=verbose)
+    elif desc['class'] == 'globalworkflow':
+        print_globalworkflow_desc(desc, verbose=verbose)
     elif desc['class'] in ['job', 'analysis']:
         print_execution_desc(desc)
     elif desc['class'] == 'user':
@@ -837,10 +962,45 @@ def get_ls_desc(desc, print_id=False):
     else:
         return desc['name'] + addendum
 
+
 def print_ls_desc(desc, **kwargs):
     print(get_ls_desc(desc, **kwargs))
 
+
+def get_ls_l_header():
+    return (BOLD() +
+            'State' + DELIMITER('   ') +
+            'Last modified' + DELIMITER('       ') +
+            'Size' + DELIMITER('      ') +
+            'Name' + DELIMITER(' (') +
+            'ID' + DELIMITER(')') +
+            ENDC())
+
+
+def print_ls_l_header():
+    print(get_ls_l_header())
+
+
+def get_ls_l_desc_fields():
+    return {
+        'id': True,
+        'class': True,
+        'folder': True,
+        'length': True,
+        'modified': True,
+        'name': True,
+        'project': True,
+        'size': True,
+        'state': True
+    }
+
+
 def get_ls_l_desc(desc, include_folder=False, include_project=False):
+    """
+    desc must have at least all the fields given by get_ls_l_desc_fields.
+    """
+    # If you make this method consume an additional field, you must add it to
+    # get_ls_l_desc_fields above.
     if 'state' in desc:
         state_len = len(desc['state'])
         if desc['state'] != 'closed':
@@ -865,13 +1025,20 @@ def get_ls_l_desc(desc, include_folder=False, include_project=False):
         size_str = get_size_str(desc['size'])
     elif 'length' in desc:
         size_str = str(desc['length']) + ' rows'
-    size_padding = ' '*(max(0, 8 - len(size_str)))
+    size_padding = ' ' * max(0, 9 - len(size_str))
+
+    return (state_str +
+            DELIMITER(' '*(8 - state_len)) + render_short_timestamp(desc['modified']) +
+            DELIMITER(' ') + size_str +
+            DELIMITER(size_padding + ' ') + name_str +
+            DELIMITER(' (') + ((desc['project'] + DELIMITER(':')) if include_project else '') + desc['id'] +
+            DELIMITER(')'))
 
-    return state_str + DELIMITER(' '*(8 - state_len)) + render_short_timestamp(desc['modified']) + DELIMITER(' ') + size_str + DELIMITER(size_padding + ' ') + name_str + DELIMITER(' (') + ((desc['project'] + DELIMITER(':')) if include_project else '') + desc['id'] + DELIMITER(')')
 
 def print_ls_l_desc(desc, **kwargs):
     print(get_ls_l_desc(desc, **kwargs))
 
+
 def get_find_executions_string(desc, has_children, single_result=False, show_outputs=True,
                                is_cached_result=False):
     '''
