@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2015 DNAnexus, Inc.
+# Copyright (C) 2013-2016 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -23,6 +23,7 @@ from __future__ import print_function, unicode_literals, division, absolute_impo
 import os, sys, json, re, collections, logging, argparse, string, itertools, subprocess, tempfile
 from functools import wraps
 from collections import namedtuple
+import pipes
 
 import dxpy
 from ..compat import USING_PYTHON2, open
@@ -269,9 +270,27 @@ class DXExecDependencyInstaller(object):
     group_pms = ("apt", "gem", "cpan", "cran", "pip")
 
     def __init__(self, executable_desc, job_desc, logger=None):
+        """
+        :param executable_desc: The description of the executable of
+            this job, which can be obtained in the response of the
+            /executable-x/describe request. This dict must contain the
+            following keys:
+                - "runSpec"
+        :type executable_desc: dict
+        :param job_desc: The description of this job, which can be
+            obtained in the response of the /job-x/describe request.
+            If ``executable_desc["runSpec"]`` has key
+            "bundledDependsByRegion", then this dict must contain the
+            following keys:
+                - "region"
+        :type job_desc: dict
+        """
         if "runSpec" not in executable_desc:
             raise DXExecDependencyError('Expected field "runSpec" to be present in executable description"')
 
+        if "bundledDependsByRegion" in executable_desc["runSpec"] and "region" not in job_desc:
+            raise DXExecDependencyError("Expected key 'region' in job description")
+
         self.exec_desc = executable_desc
         self.run_spec = executable_desc["runSpec"]
         self.job_desc = job_desc
@@ -279,20 +298,36 @@ class DXExecDependencyInstaller(object):
         self.logger = logger
 
         self.dep_groups = []
-        for dep in itertools.chain(self.run_spec.get("bundledDepends", []),
+        for dep in itertools.chain(self._get_local_bundled_dependencies(),
                                    self.run_spec.get("execDepends", []),
                                    self.run_spec.get("dependencies", [])):
             self._validate_dependency(dep)
+
+            dep_type = self._get_dependency_type(dep)
+
+            # Ignore dx-toolkit and jq from execDepends. They do not play nice
+            if dep["name"] in ("dx-toolkit", "jq", "python-argcomplete") and dep_type == "apt":
+                continue
+                
             if "stages" in dep and self.stage not in dep["stages"]:
                 self.log("Skipping dependency {} because it is inactive in stage (function) {}".format(dep["name"],
                                                                                                        self.stage))
                 continue
 
-            dep_type = self._get_dependency_type(dep)
+            
             if len(self.dep_groups) == 0 or self.dep_groups[-1]["type"] != dep_type or dep_type not in self.group_pms:
                 self.dep_groups.append({"type": dep_type, "deps": [], "index": len(self.dep_groups)})
             self.dep_groups[-1]["deps"].append(dep)
 
+    def _get_local_bundled_dependencies(self):
+        bundled_depends_by_region = self.run_spec.get("bundledDependsByRegion")
+        if bundled_depends_by_region is None:
+            return self.run_spec.get("bundledDepends", [])
+        else:
+            # Resolve the local bundled dependencies for this job.
+            job_region = self.job_desc["region"]
+            return bundled_depends_by_region[job_region]
+
     def log(self, message):
         if self.logger:
             self.logger.info(message)
@@ -385,7 +420,7 @@ class DXExecDependencyInstaller(object):
         if bundle["id"].get("$dnanexus_link", "").startswith("file-"):
             self.log("Downloading bundled file {name}".format(**bundle))
             dxpy.download_dxfile(bundle["id"], bundle["name"])
-            self.run("dx-unpack '{}'".format(bundle["name"]))
+            self.run("dx-unpack {}".format(pipes.quote(bundle["name"])))
         else:
             self.log('Skipping bundled dependency "{name}" because it does not refer to a file'.format(**bundle))
 
