@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2015 DNAnexus, Inc.
+# Copyright (C) 2013-2016 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -143,8 +143,8 @@ class ResolutionError(DXError):
         return self.msg
 
 data_obj_pattern = re.compile('^(record|gtable|applet|file|workflow)-[0-9A-Za-z]{24}$')
-hash_pattern = re.compile('^(record|gtable|app|applet|workflow|job|analysis|project|container|file)-[0-9A-Za-z]{24}$')
-nohash_pattern = re.compile('^(user|org|app|team)-')
+hash_pattern = re.compile('^(record|gtable|app|applet|workflow|globalworkflow|job|analysis|project|container|file)-[0-9A-Za-z]{24}$')
+nohash_pattern = re.compile('^(user|org|app|globalworkflow|team)-')
 jbor_pattern = re.compile('^(job|analysis)-[0-9A-Za-z]{24}:[a-zA-Z_][0-9a-zA-Z_]*$')
 
 def is_hashid(string):
@@ -697,6 +697,7 @@ def _check_resolution_needed(path, project, folderpath, entity_name, expected_cl
                 describe['project'] = dxpy.WORKSPACE_ID
         try:
             desc = dxpy.DXHTTPRequest('/' + entity_name + '/describe', describe)
+            desc = dxpy.append_underlying_workflow_describe(desc)
         except Exception as details:
             if 'project' in describe:
                 # Now try it without the hint
@@ -811,6 +812,10 @@ def _validate_resolution_output_length(path, entity_name, results, allow_mult=Fa
             return results
         if INTERACTIVE_CLI:
             print('The given path "' + path + '" resolves to the following data objects:')
+            if any(['describe' not in result for result in results]):
+                # findDataObject API call must be made to get 'describe' mappings
+                project, folderpath, entity_name = resolve_path(path, expected='entity')
+                results = _resolve_global_entity(project, folderpath, entity_name)
             choice = pick([get_ls_l_desc(result['describe']) for result in results],
                           allow_mult=allow_mult)
             if allow_mult and choice == '*':
@@ -858,6 +863,10 @@ def _resolve_global_entity(project_or_job_id, folderpath, entity_name, describe=
     mapping was provided.
 
     TODO: Inspect entity_name and conditionally treat it as a "glob" pattern.
+
+    TODO: Callers should specify exactly what fields they want, and then
+    hopefully we can avoid having a default set of fields that may be very
+    expensive
     """
     if is_job_id(project_or_job_id):
         if describe is True:
@@ -1007,8 +1016,16 @@ def resolve_multiple_existing_paths(paths):
 def resolve_existing_path(path, expected=None, ask_to_resolve=True, expected_classes=None, allow_mult=False,
                           describe=True, all_mult=False, allow_empty_string=True, visibility="either"):
     '''
+    :param expected: one of the following: "folder", "entity", or None to indicate
+                     whether the expected path is a folder, a data object, or either
+    :type expected: string or None
     :param ask_to_resolve: Whether picking may be necessary (if true, a list is returned; if false, only one result is returned)
     :type ask_to_resolve: boolean
+    :param expected_classes: A list of expected classes the entity is allowed
+                             to belong to if it is an ID (e.g. "record",
+                             "file", "job"); if None, then entity_name may be
+                             any data object class
+    :type expected_classes: list or None
     :param allow_mult: Whether to allow the user to select multiple results from the same path
     :type allow_mult: boolean
     :param describe: Input hash to describe call for the results, or True if no describe input
@@ -1032,7 +1049,9 @@ def resolve_existing_path(path, expected=None, ask_to_resolve=True, expected_cla
     Output is of the form {"id": id, "describe": describe hash} a list
     of those
 
-    TODO: Allow arbitrary flags for the describe hash.
+    TODO: Callers should specify exactly what fields they want, and then
+    hopefully we can avoid having a default set of fields that may be very
+    expensive
 
     NOTE: if expected_classes is provided and conflicts with the class
     of the hash ID, it will return None for all fields.
@@ -1119,14 +1138,48 @@ def get_app_from_path(path):
         alias = path[path.find('/') + 1:]
         path = path[:path.find('/')]
     try:
-        if alias is None:
-            desc = dxpy.DXHTTPRequest('/' + path + '/describe', {})
-        else:
-            desc = dxpy.DXHTTPRequest('/' + path + '/' + alias + '/describe', {})
-        return desc
+        return dxpy.api.app_describe(path, alias=alias)
     except dxpy.DXAPIError:
         return None
 
+def get_global_workflow_from_path(path):
+    '''
+    :param path: A string to attempt to resolve to a global workflow object
+    :type path: string
+    :returns: The describe hash of the global workflow object if found, or None otherwise
+    :rtype: dict or None
+
+    This method parses a string that is expected to perhaps refer to
+    a global workflow object.  If found, its describe hash will be returned.
+    For more information on the contents of this hash, see the API
+    documentation. [TODO: external link here]
+
+    '''
+    alias = None
+    if not path.startswith('globalworkflow-'):
+        path = 'globalworkflow-' + path
+    if '/' in path:
+        alias = path[path.find('/') + 1:]
+        path = path[:path.find('/')]
+
+    try:
+        return dxpy.api.global_workflow_describe(path, alias=alias)
+    except dxpy.DXAPIError:
+        return None
+
+def get_global_exec_from_path(path):
+    if path.startswith('app-'):
+        return get_app_from_path(path)
+    elif path.startswith('globalworkflow-'):
+        return get_global_workflow_from_path(path)
+
+    # If the path doesn't include a prefix, we must try describing
+    # as an app and, if that fails, as a global workflow
+    desc = get_app_from_path(path)
+    if not desc:
+        desc = get_global_workflow_from_path(path)
+    return desc
+
 def resolve_app(path):
     '''
     :param path: A string which is supposed to identify an app
@@ -1146,6 +1199,60 @@ def resolve_app(path):
     else:
         return desc
 
+def resolve_global_workflow(path):
+    '''
+    :param path: A string which is supposed to identify a global workflow
+    :type path: string
+    :returns: The describe hash of the global workflow object
+    :raises: :exc:`ResolutionError` if it cannot be found
+
+    *path* is expected to have one of the following forms:
+
+    - hash ID, e.g. "globalworkflow-F85Z6bQ0xku1PKY6FjGQ011J"
+    - named ID, e.g. "globalworkflow-myworkflow"
+    - named ID with alias (version or tag), e.g. "globalworkflow-myworkflow/1.2.0"
+    '''
+    desc = get_global_workflow_from_path(path)
+    if desc is None:
+        raise ResolutionError('The given path "' + path + '" could not be resolved to an accessible global workflow')
+    else:
+        return desc
+
+def resolve_global_executable(path, is_version_required=False):
+    """
+    :param path: A string which is supposed to identify a global executable (app or workflow)
+    :type path: string
+    :param is_version_required: If set to True, the path has to specify a specific version/alias, e.g. "myapp/1.0.0"
+    :type is_version_required: boolean
+    :returns: The describe hash of the global executable object (app or workflow)
+    :raises: :exc:`ResolutionError` if it cannot be found
+
+    *path* is expected to have one of the following forms:
+
+    - hash ID, e.g. "globalworkflow-F85Z6bQ0xku1PKY6FjGQ011J", "app-FBZ3f200yfzkKYyp9JkFVQ97"
+    - named ID, e.g. "app-myapp", "globalworkflow-myworkflow"
+    - named ID with alias (version or tag), e.g. "myapp/1.2.0", "myworkflow/1.2.0"
+    - named ID with prefix and with alias (version or tag), e.g. "app-myapp/1.2.0", "globalworkflow-myworkflow/1.2.0"
+    """
+    if is_version_required and "/" not in path:
+        raise ResolutionError('Version is required, e.g. "myexec/1.0.0"'.format())
+
+    # First, check if the prefix is provided, then we don't have to resolve the name
+    if path.startswith('app-'):
+        return resolve_app(path)
+    elif path.startswith('globalworkflow-'):
+        return resolve_global_workflow(path)
+
+    # If the path doesn't include a prefix, we must try describing
+    # as an app and, if that fails, as a global workflow
+    desc = get_app_from_path(path)
+    if not desc:
+        desc = get_global_workflow_from_path(path)
+    if desc is None:
+        raise ResolutionError(
+            'The given path "' + path + '" could not be resolved to an accessible global executable (app or workflow)')
+    return desc
+
 def get_exec_handler(path, alias=None):
     handler = None
     def get_handler_from_desc(desc):
@@ -1153,11 +1260,17 @@ def get_exec_handler(path, alias=None):
             return dxpy.DXApplet(desc['id'], project=desc['project'])
         elif desc['class'] == 'app':
             return dxpy.DXApp(dxid=desc['id'])
-        else:
+        elif desc['class'] == 'workflow':
             return dxpy.DXWorkflow(desc['id'], project=desc['project'])
+        elif desc['class'] == 'globalworkflow':
+            return dxpy.DXGlobalWorkflow(dxid=desc['id'])
+        else:
+            raise DXError('The executable class {} is not supported'.format(desc['class']))
+
+    # First attempt to resolve a global executable: app or global workflow
+    global_exec_desc = get_global_exec_from_path(path)
 
     if alias is None:
-        app_desc = get_app_from_path(path)
         try:
             # Look for applets and workflows
             _project, _folderpath, entity_results = resolve_existing_path(path,
@@ -1172,35 +1285,37 @@ def get_exec_handler(path, alias=None):
                 if len(entity_results) == 0:
                     entity_results = None
         except ResolutionError:
-            if app_desc is None:
+            if global_exec_desc is None:
                 raise
             else:
                 entity_results = None
 
-        if entity_results is not None and len(entity_results) == 1 and app_desc is None:
+        if entity_results is not None and len(entity_results) == 1 and global_exec_desc is None:
             handler = get_handler_from_desc(entity_results[0]['describe'])
-        elif entity_results is None and app_desc is not None:
-            handler = get_handler_from_desc(app_desc)
+        elif entity_results is None and global_exec_desc is not None:
+            handler = get_handler_from_desc(global_exec_desc)
         elif entity_results is not None:
             if not INTERACTIVE_CLI:
                 raise ResolutionError('Found multiple executables with the path ' + path)
             print('Found multiple executables with the path ' + path)
             choice_descriptions = [get_ls_l_desc(r['describe']) for r in entity_results]
-            if app_desc is not None:
-                choice_descriptions.append('app-' + app_desc['name'] + ', version ' + app_desc['version'])
+            if global_exec_desc is not None:
+                choice_descriptions.append(
+                    '{prefix}-{name}, version {version}'.format(
+                        prefix=global_exec_desc['class'],
+                        name=global_exec_desc['name'],
+                        version=global_exec_desc['version']))
             choice = pick(choice_descriptions)
             if choice < len(entity_results):
-                # all applet/workflow choices show up before the app,
+                # all applet/workflow choices show up before the global app/workflow,
                 # of which there is always at most one possible choice
                 handler = get_handler_from_desc(entity_results[choice]['describe'])
             else:
-                handler = get_handler_from_desc(app_desc)
+                handler = get_handler_from_desc(global_exec_desc)
         else:
             raise ResolutionError("No matches found for " + path)
     else:
-        if path.startswith('app-'):
-            path = path[4:]
-        handler = dxpy.DXApp(name=path, alias=alias)
+        handler = get_handler_from_desc(global_exec_desc)
     return handler
 
 def resolve_to_objects_or_project(path, all_matching_results=False):
@@ -1230,3 +1345,16 @@ def resolve_to_objects_or_project(path, all_matching_results=False):
                                   if you were attempting to refer to a project by name,
                                   please append a colon ":" to indicate that it is a project.''')
     return project, folderpath, entity_results
+
+# Generic function to parse an input key-value pair of the form '-ikey=val'
+# e.g. returns ("key", "val") in the example above
+def parse_input_keyval(keyeqval):
+    try:
+        first_eq_pos = get_first_pos_of_char('=', keyeqval)
+        if first_eq_pos == -1:
+            raise
+        name = split_unescaped('=', keyeqval)[0]
+        value = keyeqval[first_eq_pos + 1:]
+        return (name, value)
+    except:
+        raise DXCLIError('An input was found that did not conform to the syntax: -i<input name>=<input value>')
