@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014-2015 DNAnexus, Inc.
+# Copyright (C) 2014-2016 DNAnexus, Inc.
 #
 # This file is part of dx-toolkit (DNAnexus platform client libraries).
 #
@@ -26,9 +26,11 @@ import tempfile
 import shutil
 import pipes
 import dxpy
+from dxpy.utils.completer import InstanceTypesCompleter
 from dxpy_testutil import DXTestCase, check_output, temporary_project, override_environment
 import dxpy_testutil as testutil
 from dxpy.exceptions import DXJobFailureError
+from dxpy.bindings.download_all_inputs import _get_num_parallel_threads
 
 def run(command, **kwargs):
     try:
@@ -52,43 +54,35 @@ LOCAL_SCRIPTS = os.path.join(os.path.dirname(__file__), '..', 'scripts')
 LOCAL_UTILS = os.path.join(os.path.dirname(__file__), '..', 'dxpy', 'utils')
 
 
+def ignore_folders(directory, contents):
+    accepted_bin = ['dx-unpack', 'dx-unpack-file', 'dxfs', 'register-python-argcomplete',
+                    'python-argcomplete-check-easy-install-script']
+    if "test" in directory:
+        return contents
+    if "../bin" in directory:
+        return [f for f in contents if f not in accepted_bin]
+    return []
+
 def build_app_with_bash_helpers(app_dir, project_id):
     tempdir = tempfile.mkdtemp()
     try:
         updated_app_dir = os.path.join(tempdir, os.path.basename(app_dir))
         shutil.copytree(app_dir, updated_app_dir)
-        # Copy the scripts we'd like to test. These can go directly into
-        # /usr/local/bin in the guest, since the normal executables will
-        # have been installed into /usr/bin, and /usr/local/bin will
-        # appear earlier on the PATH, overriding them.
-        resources_bindir = os.path.join(updated_app_dir, 'resources', 'usr', 'local', 'bin')
-        if not os.path.exists(resources_bindir):
-            os.makedirs(resources_bindir)
-        shutil.copy(os.path.join(LOCAL_SCRIPTS, 'dx-download-all-inputs'), resources_bindir)
-        shutil.copy(os.path.join(LOCAL_SCRIPTS, 'dx-upload-all-outputs'), resources_bindir)
+        # Copy the current verion of dx-toolkit. We will build it on the worker
+        # and source this version which will overload the stock version of dx-toolkit.
+        # This we we can test all bash helpers as they would appear locally with all
+        # necessary dependencies
+        dxtoolkit_dir = os.path.join(updated_app_dir, 'resources', 'dxtoolkit')
+        local_dxtoolkit = os.path.join(os.path.dirname(__file__), '..', '..', '..')
+        shutil.copytree(local_dxtoolkit, dxtoolkit_dir, ignore=ignore_folders)
 
-        # Now copy any libraries we depend on. This is tricky to get
-        # right in general (because we will end up with some subset of
-        # the files from whatever version is installed on the worker by
-        # default, and some subset of files replaced with our custom
-        # versions here). So it might be wise to keep at a minimum the
-        # number of files that will be replaced here.
-        #
-        # In order to prevent the files in the resources bundle that go
-        # into /usr/share/dnanexus/... from being clobbered at
-        # execDepends installation time with the (older) versions from
-        # dx-toolkit, we do the following multi-stage deployment:
-        # (1) At build time, copy the files into /opt/utils_staging_area
-        # (2) Then, at runtime, copy the files into the proper place.
-        utils_staging_area = os.path.join(updated_app_dir, 'resources', 'opt', 'utils_staging_area')
-        os.makedirs(utils_staging_area)
+        # Add lines to the beginning of the job to make and use our new dx-toolkit
         preamble = []
-        for filename in ('file_load_utils.py', 'printing.py'):
-            shutil.copy(os.path.join(LOCAL_UTILS, filename), utils_staging_area)
-            cmd = "cp /opt/utils_staging_area/{f} /usr/share/dnanexus/lib/python2.7/site-packages/*/dxpy/utils;\n"
-            preamble.append(cmd.format(f=filename))
-        # Now find the applet entry point file and prepend the copy
-        # operations (step 2 above), overwriting it in place.
+        preamble.append('sudo pip install --upgrade virtualenv\n')
+        preamble.append('make -C /dxtoolkit python\n')
+        preamble.append('source /dxtoolkit/environment\n')
+        # Now find the applet entry point file and prepend the
+        # operations above, overwriting it in place.
         dxapp_json = json.load(open(os.path.join(app_dir, 'dxapp.json')))
         if dxapp_json['runSpec']['interpreter'] != 'bash':
             raise Exception('Sorry, I only know how to patch bash apps for remote testing')
@@ -417,7 +411,7 @@ class TestDXBashHelpersBenchmark(DXTestCase):
 
 class TestDXJobutilAddOutput(DXTestCase):
     dummy_hash = "123456789012345678901234"
-    data_obj_classes = ['file', 'record', 'gtable', 'applet', 'workflow']
+    data_obj_classes = ['file', 'record', 'applet', 'workflow']
     dummy_ids = [obj_class + '-' + dummy_hash for obj_class in data_obj_classes]
     dummy_job_id = "job-" + dummy_hash
     dummy_analysis_id = "analysis-123456789012345678901234"
@@ -532,7 +526,21 @@ class TestDXJobutilNewJob(DXTestCase):
     def tearDownClass(cls):
         dxpy.api.project_destroy(cls.aux_project.get_id(), {})
 
-    def test_dx_jobutil_new_job(self):
+    def assertNewJobInputHash(self, cmd_snippet, arguments_hash):
+        cmd = "dx-jobutil-new-job entrypointname " + cmd_snippet + " --test"
+        expected_job_input = {"function": "entrypointname", "input": {}}
+        env = override_environment(DX_JOB_ID="job-000000000000000000000001", DX_WORKSPACE_ID=self.project)
+        output = run(cmd, env=env)
+        expected_job_input.update(arguments_hash)
+        self.assertEqual(json.loads(output), expected_job_input)
+
+    def assertNewJobError(self, cmd_snippet, exit_code):
+        cmd = "dx-jobutil-new-job entrypointname " + cmd_snippet + " --test"
+        env = override_environment(DX_JOB_ID="job-000000000000000000000001", DX_WORKSPACE_ID=self.project)
+        with self.assertSubprocessFailure(exit_code=exit_code):
+            run(cmd, env=env)
+
+    def test_input(self):
         first_record = dxpy.new_dxrecord(name="first_record")
         second_record = dxpy.new_dxrecord(name="second_record")
         dxpy.new_dxrecord(name="duplicate_name_record")
@@ -589,9 +597,6 @@ class TestDXJobutilNewJob(DXTestCase):
             ("-ifoo:string=first_record", {"foo": "first_record"}),
             ('-ifoo:hash=\'{"a": "b"}\'', {"foo": {"a": "b"}}),
             ('-ifoo:hash=\'["a", "b"]\'', {"foo": ["a", "b"]}),
-            ("-ifoo:file=first_record", None),  # Error
-            ("-ifoo:int=foo", None),  # Error
-            ("-ifoo:int=24.5", None),  # Error
 
             # Array inputs
 
@@ -610,16 +615,70 @@ class TestDXJobutilNewJob(DXTestCase):
             ("-ifoo:array:int=24", {"foo": 24}),
             ("-ifoo:array:record=first_record", {"foo": dxpy.dxlink(first_record.get_id(), self.project)}),
         )
-        env = override_environment(DX_JOB_ID="job-000000000000000000000001",
-                                   DX_WORKSPACE_ID=self.project)
+
         for cmd_snippet, expected_input_hash in test_cases:
-            cmd = "dx-jobutil-new-job " + cmd_snippet + " entrypointname --test"
-            if expected_input_hash is None:
-                with self.assertSubprocessFailure(exit_code=1):
-                    run(cmd, env=env)
-            else:
-                output = run(cmd, env=env)
-                self.assertEqual(json.loads(output), {"input": expected_input_hash, "function": "entrypointname"})
+            arguments_hash = {"input": expected_input_hash}
+            self.assertNewJobInputHash(cmd_snippet, arguments_hash)
+
+    def test_bad_input(self):
+        # testing some erroneous input
+        self.assertNewJobError("-ifoo:file=first_record", 1)
+        self.assertNewJobError("-ifoo:int=foo", 1)
+        self.assertNewJobError("-ifoo:int=24.5", 1)
+
+    def test_job_arguments(self):
+        test_arguments = (
+            # name - string
+            ("--name JobName", {"name": "JobName"}),
+            # depends-on - array of strings
+            ("--depends-on foo bar baz", {"dependsOn": ["foo", "bar", "baz"]}),
+            # instance type: single instance - string
+            ("--instance-type foo_bar_baz",
+             {"systemRequirements": {"entrypointname": { "instanceType": "foo_bar_baz" }}}),
+            # instance type: mapping
+            ("--instance-type " +
+                pipes.quote(json.dumps({"main": "mem2_hdd2_x2" , "other_function": "mem2_hdd2_x1" })),
+                {"systemRequirements": {"main": { "instanceType": "mem2_hdd2_x2" },
+                                        "other_function": { "instanceType": "mem2_hdd2_x1" }}}),
+            # properties - mapping
+            ("--property foo=foo_value --property bar=bar_value",
+                {"properties": {"foo": "foo_value", "bar": "bar_value"}}),
+            # tags - array of strings
+            ("--tag foo --tag bar --tag baz", {"tags": ["foo", "bar", "baz"]}),
+        )
+        for cmd_snippet, arguments_hash in test_arguments:
+            self.assertNewJobInputHash(cmd_snippet, arguments_hash)
+
+    def test_extra_arguments(self):
+        cmd_snippet = "--extra-args " + pipes.quote(
+            json.dumps({"details": {"d1": "detail1", "d2": 1234}, "foo": "foo_value"}))
+        arguments_hash = {"details": {"d1": "detail1", "d2": 1234}, "foo": "foo_value"}
+        self.assertNewJobInputHash(cmd_snippet, arguments_hash)
+
+        # override previously specified args
+        cmd_snippet = "--name JobName --extra-args " + pipes.quote(json.dumps({"name": "FinalName"}))
+        arguments_hash = {"name": "FinalName"}
+        self.assertNewJobInputHash(cmd_snippet, arguments_hash)
+
+    def test_bad_arguments(self):
+        # empty name
+        self.assertNewJobError("--name", exit_code=2)
+        # property not in key=value format
+        self.assertNewJobError("--property foo", exit_code=3)
+        # extra-args not in key=value format
+        self.assertNewJobError("--extra-args argument", exit_code=3)
+
+
+class TestDXBashHelperMethods(unittest.TestCase):
+    def test_limit_threads(self):
+        ''' Tests that the number of threads used for downloading inputs in parallel is limited '''
+        instance_types = InstanceTypesCompleter().instance_types
+        max_threads = 8
+        for inst in instance_types.values():
+            num_threads = _get_num_parallel_threads(max_threads, inst.CPU_Cores, inst.Memory_GB*1024)
+            self.assertTrue(num_threads >= 1 and num_threads <= max_threads)
+            self.assertTrue(num_threads <= inst.CPU_Cores)
+            self.assertTrue(num_threads*1200 <= inst.Memory_GB*1024 or num_threads == 1)
 
 
 if __name__ == '__main__':
